/*
 * inverter.c
 *
 *  Created on: 19.09.2016
 *      Author: dehottgw
 */


#include "inverterCtrl.h"

#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>
#include <msp430g2553.h>
#include <isr_compat.h>


#define NUM_OF_SINE_VALUES 21


volatile uint8_t pulseWidthIdx = 0;
volatile bool direction = true;

volatile uint32_t tick;


ISR(TIMER1_A0, TA1_ISR_Ovrfl) {
	tick++;

	if (direction) {
		if (pulseWidthIdx == 0) {
			P2OUT |= BIT3;
		}
		if (pulseWidthIdx == (NUM_OF_SINE_VALUES / 3)) {
			P2OUT |= BIT4;
		}
		if (pulseWidthIdx == (NUM_OF_SINE_VALUES / 3 * 2)) {
			P2OUT |= BIT5;
		}
	} else {
		if (pulseWidthIdx == 0) {
			P2OUT |= BIT4;
		}
		if (pulseWidthIdx == (NUM_OF_SINE_VALUES / 3)) {
			P2OUT |= BIT3;
		}
		if (pulseWidthIdx == (NUM_OF_SINE_VALUES / 3 * 2)) {
			P2OUT |= BIT5;
		}
	}

	pulseWidthIdx++;
	if (pulseWidthIdx >= NUM_OF_SINE_VALUES * 2) {
		pulseWidthIdx = 0;
	}
}




void inverterCtrlInit() {
	// reset, start, stop
	P2SEL &= ~(BIT2 | BIT3 | BIT4 | BIT5);
	P2OUT &= ~(BIT2 | BIT3 | BIT4 | BIT5);
	P2DIR |= BIT2 | BIT3 | BIT4 | BIT5;

	// start mark
	P2OUT &= ~BIT0;
	P2DIR |= BIT0;
	P2SEL |= BIT0;

	// smclk out
	P1DIR |= BIT4;
	P1SEL |= BIT4;


	tick = 0;

	TA1CTL = MC_0;
	TA1CCTL0 = CCIE | OUTMOD_4;
	TA1CCR0 = 0;

	P2OUT &= ~BIT2;

	volatile uint32_t w;
	for (w = 0; w < 100000; w++);

	// remove reset from PWNs
    P2OUT |= BIT2;

	for (w = 0; w < 100000; w++);

	TA1CTL = MC_1 | ID_0 | TASSEL_2 | TACLR;


}



void inverterCtrlSetFrequency(uint16_t f) {
	float ff = (float)f;
	float slotLength = 4 / (ff * 400e-9 * NUM_OF_SINE_VALUES);
	uint16_t sl = (uint16_t)slotLength;

	__disable_interrupt();
	TA1CTL = MC_0;
	P2OUT &= ~(BIT3 | BIT4 | BIT5);
	TA1CCR0 = sl;
	pulseWidthIdx = 0;
	TA1CTL = MC_1 | ID_0 | TASSEL_2 | TACLR;
	__enable_interrupt();
}
